//FBstring portion of fbc++.h

#include "fb.h"


/***************************      STRINGS     ***************************/

class FBstring {
  /* The rtlib keeps its own list of temporary string descriptors,
   * and will pass back temporary strings with temporary descriptors.
   * Whenever you pass most functions a FBstring*, it will delete the
   * string data if the string is marked as temp, and the descriptor
   * if it's a rtlib-allocated temp one.
   */

  /* Special rule: if a function returns a string, you can not throw it away!
     Otherwise you have a memory leak 
  */
 public:
  FBSTRING str;

  FBstring() { 
    str.data = NULL;
    str.len = 0;
    str.size = 0;
    //fb_StrInit(&str, -1, "", 1, 0);
  }

  FBstring(const char *text) {
    fb_StrInit(&str, -1, (void *)text, /*strlen((char *)text)*/ 0, 0);
    //printf("<%p %p newc=%s>", this, str.data, str.data);
  }
  
  FBstring(const FBstring &rhs) {
    fb_StrInit(&str, -1, (void *)&rhs.str, -1, 0);
    //printf("<%p %p news=%s>", this, str.data, str.data);
  }
  

  FBstring(int rhs) {
    fb_StrInit(&str, -1, fb_IntToStr(rhs), -1, 0);
    //printf("<%p %p newi=%s>", this, str.data, str.data);
  }
  
  ~FBstring() {
    /*
    if (str.data)
      printf("<%p %p del=%s>", this, str.data, str.data);
    else
      printf("<%p - del>", this);
    */

    //This doesn't attempt to free a temporary descriptor. Such a string should never have its destructor called.
    fb_StrDelete(&str);
  }
  
  /*
  FBSTRING *operator&() {
    return &str;
  }
  */

  int len() const {
    //this deletes temp strings
    return fb_StrLen((void *)&str, -1);
  }

  //call this to get a value that can be returned from a function
  FBSTRING *ret() {
    return fb_StrAllocTempResult(&str);
  }

  FBstring &operator=(const FBstring &rhs) {
    //if (this != &rhs) {
    fb_StrAssign(&str, -1, (void *)&rhs.str, -1, 0);
    //}
  }

  FBstring &operator=(const char *rhs) {
    //if (this != &rhs) {
    fb_StrAssign(&str, -1, (void *)rhs, 0, 0);
    //}
  }

  FBstring &operator+=(const FBstring &rhs) {
    fb_StrConcatAssign(&str, -1, (void *)&rhs.str, -1, 0);
    return *this;
  }
};

int operator==(const FBstring &lhs, const FBstring &rhs) {
  return !fb_StrCompare((void *)&lhs.str, -1, (void *)&rhs.str, -1);
}

int operator==(const FBstring &lhs, const char *rhs) {
  return !fb_StrCompare((void *)&lhs.str, -1, (void *)rhs, 0);
}

int operator==(const char *lhs, const FBstring &rhs) {
  return !fb_StrCompare((void *)lhs, 0, (void *)&rhs.str, -1);
}

FBstring &operator+(int lhs, const FBstring &rhs) {
  //printf("<i+s=%s>", rhs.str.data);
  return *(FBstring *)fb_StrConcat(fb_hStrAllocTmpDesc(), fb_IntToStr(lhs), -1, (void *)&rhs.str, -1);
}

FBstring &operator+(const FBstring &lhs, const FBstring &rhs) {
  //printf("str1: ptr %p %x %d\n", lhs.str.data, lhs.str.len, lhs.str.size);
  //printf("str2: ptr %p %x %d\n", rhs.str.data, rhs.str.len, rhs.str.size);

  if (FB_ISTEMP(&lhs.str))
    //Reuse this temp string (ConcatAssign doesn't delete it)
    return *(FBstring *)fb_StrConcatAssign((void *)&lhs.str, -1, (void *)&rhs.str, -1, 0);
  else
    //Should be able to pass NULL as first pointer, but that segfaults. Bug filed and fixed in CVS
    // return *(FBstring *)fb_StrConcat(NULL, (void *)&str, -1, (void *)&rhs.str, -1);
    return *(FBstring *)fb_StrConcat(fb_hStrAllocTmpDesc(), (void *)&lhs.str, -1, (void *)&rhs.str, -1);
}

FBstring &operator+(const FBstring &lhs, const char *rhs) {
  if (FB_ISTEMP(&lhs.str))
    //Reuse this temp string (ConcatAssign doesn't delete it)
    return *(FBstring *)fb_StrConcatAssign((void *)&lhs.str, -1, (void *)rhs, 0, 0);
  else
    return *(FBstring *)fb_StrConcat(fb_hStrAllocTmpDesc(), (void *)&lhs.str, -1, (void *)rhs, 0);
}

FBstring &operator+(const FBstring &lhs, int rhs) {
  if (FB_ISTEMP(&lhs.str))
    //Reuse this temp string (ConcatAssign doesn't delete it)
    return *(FBstring *)fb_StrConcatAssign((void *)&lhs.str, -1, fb_IntToStr(rhs), -1, 0);
  else
    return *(FBstring *)fb_StrConcat(fb_hStrAllocTmpDesc(), (void *)&lhs.str, -1, fb_IntToStr(rhs), -1);
}

FBstring &operator+(const char *lhs, const FBstring &rhs) {
  //printf("<c+s=%s>", rhs.str.data);
  return *(FBstring *)fb_StrConcat(fb_hStrAllocTmpDesc(), (void *)lhs, 0, (void *)&rhs.str, -1);
}
